import torch

from .normal import Normal
from .transformed_distribution import TransformedDistribution


class LogNormal(TransformedDistribution):
    def __init__(self, loc, scale, validate_args=None):
        base_dist = Normal(loc=loc, scale=scale, validate_args=validate_args)
        super().__init__(
            base_distribution=base_dist,
            transforms=[torch.distributions.ExpTransform()],
            validate_args=validate_args,
        )



import torch
from torch.distributions import AffineTransform, ExpTransform
from torch.distributions.utils import broadcast_all

from .exponential import Exponential
from .transformed_distribution import TransformedDistribution


class Pareto(torch.distributions.Pareto, TransformedDistribution):
    def __init__(self, scale, alpha, validate_args=None):
        self.scale, self.alpha = broadcast_all(scale, alpha)
        base_dist = Exponential(self.alpha, validate_args=validate_args)
        transforms = [ExpTransform(), AffineTransform(loc=0, scale=self.scale)]
        TransformedDistribution.__init__(
            self, base_dist, transforms, validate_args=validate_args
        )



import torch
from torch.distributions import constraints

from .survival_distribution import SurvivalDistribution


class Uniform(torch.distributions.Uniform, SurvivalDistribution):
    def logsf(self, value):
        if self._validate_args:
            self._validate_sample(value)
        return (self.high - value).log() - (self.high - self.low).log()

    def isf(self, u):
        if self._validate_args:
            assert constraints.unit_interval.check(u).all()
        result = self.high - u * (self.high - self.low)
        return result

    def _new_tensor(self, shape):
        return self.low.new(shape)
